@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
@@ -38,22 +36,18 @@ enum {
 
 static GParamSpec *obj_props[PROP_LAST];
 
-enum {
-  NEXT_PAGE,
-  LAST_SIGNAL,
-};
-
-static guint signals[LAST_SIGNAL];
-
 struct _GisAssistantPrivate
 {
   GtkWidget *forward;
+  GtkWidget *accept;
+  GtkWidget *skip;
   GtkWidget *back;
   GtkWidget *cancel;
-  GtkWidget *progress_indicator;
+
   GtkWidget *main_layout;
   GtkWidget *spinner;
   GtkWidget *titlebar;
+  GtkWidget *title;
   GtkWidget *stack;
 
   GList *pages;
@@ -84,42 +78,14 @@ widget_destroyed (GtkWidget    *widget,
 }
 
 static void
-gis_assistant_switch_to (GisAssistant          *assistant,
-                         GisAssistantDirection  direction,
-                         GisPage               *page)
+switch_to (GisAssistant          *assistant,
+           GisPage               *page)
 {
   GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
 
   gtk_stack_set_visible_child (GTK_STACK (priv->stack), GTK_WIDGET (page));
 }
 
-static void
-on_apply_done (GisPage *page,
-               gboolean valid,
-               gpointer user_data)
-{
-  GisAssistant *assistant = GIS_ASSISTANT (user_data);
-  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
-
-  if (valid)
-    g_signal_emit (assistant, signals[NEXT_PAGE], 0,
-                   priv->current_page);
-
-  g_object_unref (assistant);
-}
-
-void
-gis_assistant_next_page (GisAssistant *assistant)
-{
-  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
-  if (priv->current_page)
-    gis_page_apply_begin (priv->current_page, on_apply_done,
-                          g_object_ref (assistant));
-  else
-    g_signal_emit (assistant, signals[NEXT_PAGE], 0,
-                   priv->current_page);
-}
-
 static inline gboolean
 should_show_page (GList *l)
 {
@@ -137,10 +103,31 @@ find_next_page (GisPage *page)
 }
 
 static void
-gis_assistant_real_next_page (GisAssistant *assistant,
-                              GisPage      *page)
+switch_to_next_page (GisAssistant *assistant)
 {
-  gis_assistant_switch_to (assistant, GIS_ASSISTANT_NEXT, find_next_page (page));
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+  switch_to (assistant, find_next_page (priv->current_page));
+}
+
+static void
+on_apply_done (GisPage *page,
+               gboolean valid,
+               gpointer user_data)
+{
+  GisAssistant *assistant = GIS_ASSISTANT (user_data);
+
+  if (valid)
+    switch_to_next_page (assistant);
+}
+
+void
+gis_assistant_next_page (GisAssistant *assistant)
+{
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+  if (priv->current_page)
+    gis_page_apply_begin (priv->current_page, on_apply_done, assistant);
+  else
+    switch_to_next_page (assistant);
 }
 
 static GisPage *
@@ -158,43 +145,32 @@ gis_assistant_previous_page (GisAssistant *assistant)
 {
   GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   g_return_if_fail (priv->current_page != NULL);
-  gis_assistant_switch_to (assistant, GIS_ASSISTANT_PREV, find_prev_page (priv->current_page));
+  switch_to (assistant, find_prev_page (priv->current_page));
 }
 
 static void
-remove_from_progress_indicator (GtkWidget *widget,
-                                gpointer   user_data)
+set_suggested_action_sensitive (GtkWidget *widget,
+                                gboolean   sensitive)
 {
-  GisAssistantPrivate *priv = user_data;
-  gtk_container_remove (GTK_CONTAINER (priv->progress_indicator), widget);
+  gtk_widget_set_sensitive (widget, sensitive);
+  if (sensitive)
+    gtk_style_context_add_class (gtk_widget_get_style_context (widget), "suggested-action");
+  else
+    gtk_style_context_remove_class (gtk_widget_get_style_context (widget), "suggested-action");
 }
 
 static void
-update_progress_indicator (GisAssistant *assistant)
+set_navigation_button (GisAssistant *assistant,
+                       GtkWidget    *widget)
 {
   GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
-  GList *l;
 
-  gtk_container_foreach (GTK_CONTAINER (priv->progress_indicator),
-                         remove_from_progress_indicator, priv);
-
-  for (l = priv->pages; l != NULL; l = l->next)
-    {
-      GisPage *page = GIS_PAGE (l->data);
-      GtkWidget *label = gtk_label_new ("•");
-
-      if (page != priv->current_page)
-        {
-          GtkStyleContext *context = gtk_widget_get_style_context (label);
-          gtk_style_context_add_class (context, "dim-label");
-        }
-
-      gtk_container_add (GTK_CONTAINER (priv->progress_indicator), label);
-      gtk_widget_show (label);
-    }
+  gtk_widget_set_visible (priv->forward, (widget == priv->forward));
+  gtk_widget_set_visible (priv->accept, (widget == priv->accept));
+  gtk_widget_set_visible (priv->skip, (widget == priv->skip));
 }
 
-static void
+void
 update_navigation_buttons (GisAssistant *assistant)
 {
   GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
@@ -216,20 +192,26 @@ update_navigation_buttons (GisAssistant *assistant)
     }
   else
     {
-      gboolean can_go_forward, is_first_page;
-
-      gtk_widget_show (priv->back);
-      gtk_widget_show (priv->forward);
+      gboolean is_first_page;
+      GtkWidget *next_widget;
 
       is_first_page = (page_priv->link->prev == NULL);
       gtk_widget_set_visible (priv->back, !is_first_page);
 
-      can_go_forward = gis_page_get_complete (page);
-      gtk_widget_set_sensitive (priv->forward, can_go_forward);
-      if (can_go_forward)
-        gtk_style_context_add_class (gtk_widget_get_style_context (priv->forward), "suggested-action");
+      if (gis_page_get_needs_accept (page))
+        next_widget = priv->accept;
       else
-        gtk_style_context_remove_class (gtk_widget_get_style_context (priv->forward), "suggested-action");
+        next_widget = priv->forward;
+
+      if (gis_page_get_complete (page)) {
+        set_suggested_action_sensitive (next_widget, TRUE);
+        set_navigation_button (assistant, next_widget);
+      } else if (gis_page_get_skippable (page)) {
+        set_navigation_button (assistant, priv->skip);
+      } else {
+        set_suggested_action_sensitive (next_widget, FALSE);
+        set_navigation_button (assistant, next_widget);
+      }
     }
 }
 
@@ -256,8 +238,8 @@ update_titlebar (GisAssistant *assistant)
 {
   GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
 
-  gtk_header_bar_set_title (GTK_HEADER_BAR (priv->titlebar),
-                            gis_assistant_get_title (assistant));
+  gtk_label_set_label (GTK_LABEL (priv->title),
+                       gis_assistant_get_title (assistant));
 }
 
 static void
@@ -298,8 +280,6 @@ gis_assistant_add_page (GisAssistant *assistant,
 
   if (priv->current_page->assistant_priv->link == link->prev)
     update_navigation_buttons (assistant);
-
-  update_progress_indicator (assistant);
 }
 
 GisPage *
@@ -371,7 +351,6 @@ update_current_page (GisAssistant *assistant,
   update_titlebar (assistant);
   update_applying_state (assistant);
   update_navigation_buttons (assistant);
-  update_progress_indicator (assistant);
   gis_page_shown (page);
 }
 
@@ -394,6 +373,8 @@ gis_assistant_locale_changed (GisAssistant *assistant)
   GList *l;
 
   gtk_button_set_label (GTK_BUTTON (priv->forward), _("_Next"));
+  gtk_button_set_label (GTK_BUTTON (priv->accept), _("_Accept"));
+  gtk_button_set_label (GTK_BUTTON (priv->skip), _("_Skip"));
   gtk_button_set_label (GTK_BUTTON (priv->back), _("_Previous"));
   gtk_button_set_label (GTK_BUTTON (priv->cancel), _("_Cancel"));
 
@@ -422,6 +403,9 @@ gis_assistant_init (GisAssistant *assistant)
                     G_CALLBACK (current_page_changed), assistant);
 
   g_signal_connect (priv->forward, "clicked", G_CALLBACK (go_forward), assistant);
+  g_signal_connect (priv->accept, "clicked", G_CALLBACK (go_forward), assistant);
+  g_signal_connect (priv->skip, "clicked", G_CALLBACK (go_forward), assistant);
+
   g_signal_connect (priv->back, "clicked", G_CALLBACK (go_backward), assistant);
   g_signal_connect (priv->cancel, "clicked", G_CALLBACK (do_cancel), assistant);
 
@@ -457,18 +441,19 @@ gis_assistant_class_init (GisAssistantClass *klass)
   gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/org/gnome/initial-setup/gis-assistant.ui");
 
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, forward);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, accept);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, skip);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, back);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, cancel);
-  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, progress_indicator);
+
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, main_layout);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, spinner);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, titlebar);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, title);
   gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, stack);
 
   gobject_class->get_property = gis_assistant_get_property;
 
-  klass->next_page = gis_assistant_real_next_page;
-
   obj_props[PROP_TITLE] =
     g_param_spec_string ("title",
                          "", "",
@@ -476,20 +461,4 @@ gis_assistant_class_init (GisAssistantClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class, PROP_LAST, obj_props);
-
-  /**
-   * GisAssistant::next-page:
-   * @assistant: the #GisAssistant
-   * @page: the page we're leaving
-   *
-   * The ::next-page signal is emitted when we're leaving
-   * a page, allowing a page to do something when it's left.
-   */
-  signals[NEXT_PAGE] =
-    g_signal_new ("next-page",
-                  G_TYPE_FROM_CLASS (gobject_class),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (GisAssistantClass, next_page),
-                  NULL, NULL, NULL,
-                  G_TYPE_NONE, 1, GIS_TYPE_PAGE);
 }
