@@ -13,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
@@ -38,43 +36,40 @@
 #include <cheese-gtk.h>
 #endif
 
-#include <egg-list-box.h>
-
 #include "pages/language/gis-language-page.h"
+#include "pages/region/gis-region-page.h"
 #include "pages/keyboard/gis-keyboard-page.h"
 #include "pages/eulas/gis-eula-pages.h"
 #include "pages/network/gis-network-page.h"
-#include "pages/account/gis-account-page.h"
-#include "pages/location/gis-location-page.h"
+#include "pages/timezone/gis-timezone-page.h"
 #include "pages/goa/gis-goa-page.h"
+#include "pages/account/gis-account-pages.h"
+#include "pages/password/gis-password-page.h"
 #include "pages/summary/gis-summary-page.h"
 
-/* main {{{1 */
-
-static gboolean force_new_user_mode;
-static const gchar *system_setup_pages[] = {
-    "account",
-    "location"
-};
+static gboolean force_existing_user_mode;
 
 typedef void (*PreparePage) (GisDriver *driver);
 
 typedef struct {
-    const gchar *page_id;
-    PreparePage prepare_page_func;
+  const gchar *page_id;
+  PreparePage prepare_page_func;
+  gboolean new_user_only;
 } PageData;
 
-#define PAGE(name) { #name, gis_prepare_ ## name ## _page }
+#define PAGE(name, new_user_only) { #name, gis_prepare_ ## name ## _page, new_user_only }
 
 static PageData page_table[] = {
-  PAGE (language),
-  PAGE (keyboard),
-  PAGE (eula),
-  PAGE (network),
-  PAGE (account),
-  PAGE (location),
-  PAGE (goa),
-  PAGE (summary),
+  PAGE (language, FALSE),
+  PAGE (region,   FALSE),
+  PAGE (keyboard, FALSE),
+  PAGE (eula,     FALSE),
+  PAGE (network,  FALSE),
+  PAGE (timezone, TRUE),
+  PAGE (goa,      FALSE),
+  PAGE (account,  TRUE),
+  PAGE (password,  TRUE),
+  PAGE (summary,  FALSE),
   { NULL },
 };
 
@@ -86,6 +81,7 @@ should_skip_page (GisDriver    *driver,
                   gchar       **skip_pages)
 {
   guint i = 0;
+
   /* check through our skip pages list for pages we don't want */
   if (skip_pages) {
     while (skip_pages[i]) {
@@ -94,21 +90,6 @@ should_skip_page (GisDriver    *driver,
       i++;
     }
   }
-
-  switch (gis_driver_get_mode (driver)) {
-  case GIS_DRIVER_MODE_EXISTING_USER:
-    i = 0;
-    while (i < G_N_ELEMENTS (system_setup_pages)) {
-      if (g_strcmp0 (system_setup_pages[i], page_id) == 0)
-        return TRUE;
-      i++;
-    }
-    break;
-  case GIS_DRIVER_MODE_NEW_USER:
-  default:
-    break;
-  }
-
   return FALSE;
 }
 
@@ -159,6 +140,7 @@ rebuild_pages_cb (GisDriver *driver)
   GisAssistant *assistant;
   GisPage *current_page;
   gchar **skip_pages;
+  gboolean is_new_user;
 
   assistant = gis_driver_get_assistant (driver);
   current_page = gis_assistant_get_current_page (assistant);
@@ -177,35 +159,27 @@ rebuild_pages_cb (GisDriver *driver)
     ++page_data;
   }
 
-  for (; page_data->page_id != NULL; ++page_data)
-    if (!should_skip_page (driver, page_data->page_id, skip_pages))
-      page_data->prepare_page_func (driver);
+  is_new_user = (gis_driver_get_mode (driver) == GIS_DRIVER_MODE_NEW_USER);
+  for (; page_data->page_id != NULL; ++page_data) {
+    if (page_data->new_user_only && !is_new_user)
+      continue;
+
+    if (should_skip_page (driver, page_data->page_id, skip_pages))
+      continue;
+
+    page_data->prepare_page_func (driver);
+  }
 
   g_strfreev (skip_pages);
 }
 
-static gboolean
-is_running_as_gnome_initial_setup_user (void)
-{
-  struct passwd pw, *pwp;
-  char buf[4096];
-
-  getpwnam_r ("gnome-initial-setup", &pw, buf, sizeof (buf), &pwp);
-  if (pwp == NULL)
-    return FALSE;
-
-  return pw.pw_uid == getuid ();
-}
-
 static GisDriverMode
 get_mode (void)
 {
-  if (force_new_user_mode)
-    return GIS_DRIVER_MODE_NEW_USER;
-  else if (is_running_as_gnome_initial_setup_user ())
-    return GIS_DRIVER_MODE_NEW_USER;
-  else
+  if (force_existing_user_mode)
     return GIS_DRIVER_MODE_EXISTING_USER;
+  else
+    return GIS_DRIVER_MODE_NEW_USER;
 }
 
 int
@@ -216,8 +190,8 @@ main (int argc, char *argv[])
   GOptionContext *context;
 
   GOptionEntry entries[] = {
-    { "force-new-user", 0, 0, G_OPTION_ARG_NONE, &force_new_user_mode,
-      _("Force new user mode"), NULL },
+    { "existing-user", 0, 0, G_OPTION_ARG_NONE, &force_existing_user_mode,
+      _("Force existing user mode"), NULL },
     { NULL }
   };
 
@@ -243,8 +217,6 @@ main (int argc, char *argv[])
   }
 #endif
 
-  g_type_ensure (EGG_TYPE_LIST_BOX);
-
   driver = gis_driver_new (get_mode ());
   g_signal_connect (driver, "rebuild-pages", G_CALLBACK (rebuild_pages_cb), NULL);
   status = g_application_run (G_APPLICATION (driver), argc, argv);
@@ -253,6 +225,3 @@ main (int argc, char *argv[])
   g_option_context_free (context);
   return status;
 }
-
-/* Epilogue {{{1 */
-/* vim: set foldmethod=marker: */
