@@ -13,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
@@ -35,15 +33,24 @@
 #include <libgnome-desktop/gnome-languages.h>
 
 #include "cc-common-language.h"
-#include "cc-util.h"
 
 #include <glib-object.h>
 
-#include <egg-list-box.h>
+struct _CcLanguageChooserPrivate
+{
+        GtkWidget *filter_entry;
+        GtkWidget *language_list;
+	GHashTable *langs;
 
-G_DEFINE_TYPE (CcLanguageChooser, cc_language_chooser, GTK_TYPE_BIN);
+        GtkWidget *scrolled_window;
+        GtkWidget *no_results;
+        GtkWidget *more_item;
 
-#define GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CC_TYPE_LANGUAGE_CHOOSER, CcLanguageChooserPrivate))
+        gboolean showing_extra;
+        gchar *language;
+};
+typedef struct _CcLanguageChooserPrivate CcLanguageChooserPrivate;
+G_DEFINE_TYPE_WITH_PRIVATE (CcLanguageChooser, cc_language_chooser, GTK_TYPE_BOX);
 
 enum {
         PROP_0,
@@ -54,17 +61,6 @@ enum {
 
 static GParamSpec *obj_props[PROP_LAST];
 
-struct _CcLanguageChooserPrivate
-{
-        GtkWidget *no_results;
-        GtkWidget *more_item;
-        GtkWidget *filter_entry;
-        GtkWidget *language_list;
-        gboolean showing_extra;
-        gchar **filter_words;
-        gchar *language;
-};
-
 typedef struct {
         GtkWidget *box;
         GtkWidget *checkmark;
@@ -111,33 +107,29 @@ language_widget_free (gpointer data)
 
 static GtkWidget *
 language_widget_new (const char *locale_id,
+		     const char *lang,
                      gboolean    is_extra)
 {
         gchar *locale_name, *locale_current_name, *locale_untranslated_name;
-        GtkWidget *checkmark;
         LanguageWidget *widget = g_new0 (LanguageWidget, 1);
 
-        locale_name = gnome_get_language_from_locale (locale_id, locale_id);
-        locale_current_name = gnome_get_language_from_locale (locale_id, NULL);
-        locale_untranslated_name = gnome_get_language_from_locale (locale_id, "C");
+        locale_name = gnome_get_language_from_locale (lang, locale_id);
+        locale_current_name = gnome_get_language_from_locale (lang, NULL);
+        locale_untranslated_name = gnome_get_language_from_locale (lang, "C");
 
         widget->box = padded_label_new (locale_name);
+        gtk_widget_set_halign (widget->box, GTK_ALIGN_FILL);
+        gtk_widget_set_margin_start (widget->box, 10);
+        gtk_widget_set_margin_end (widget->box, 10);
         widget->locale_id = g_strdup (locale_id);
         widget->locale_name = locale_name;
         widget->locale_current_name = locale_current_name;
         widget->locale_untranslated_name = locale_untranslated_name;
         widget->is_extra = is_extra;
 
-        /* We add a check on each side of the label to keep it centered. */
-        checkmark = gtk_image_new_from_icon_name ("object-select-symbolic", GTK_ICON_SIZE_MENU);
-        gtk_widget_show (checkmark);
-        gtk_widget_set_opacity (checkmark, 0.0);
-        gtk_box_pack_start (GTK_BOX (widget->box), checkmark, FALSE, FALSE, 0);
-        gtk_box_reorder_child (GTK_BOX (widget->box), checkmark, 0);
-
         widget->checkmark = gtk_image_new_from_icon_name ("object-select-symbolic", GTK_ICON_SIZE_MENU);
-        gtk_box_pack_start (GTK_BOX (widget->box), widget->checkmark,
-                            FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (widget->box), widget->checkmark, TRUE, TRUE, 0);
+	gtk_widget_set_halign (widget->checkmark, GTK_ALIGN_END);
         gtk_widget_show (widget->checkmark);
 
         g_object_set_data_full (G_OBJECT (widget->box), "language-widget", widget,
@@ -147,13 +139,17 @@ language_widget_new (const char *locale_id,
 }
 
 static void
-sync_checkmark (GtkWidget *child,
+sync_checkmark (GtkWidget *row,
                 gpointer   user_data)
 {
-        LanguageWidget *widget = get_language_widget (child);
+        GtkWidget *child;
+        LanguageWidget *widget;
         gchar *locale_id;
         gboolean should_be_visible;
 
+        child = gtk_bin_get_child (GTK_BIN (row));
+        widget = get_language_widget (child);
+
         if (widget == NULL)
                 return;
 
@@ -165,7 +161,7 @@ sync_checkmark (GtkWidget *child,
 static void
 sync_all_checkmarks (CcLanguageChooser *chooser)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
 
         gtk_container_foreach (GTK_CONTAINER (priv->language_list),
                                sync_checkmark, priv->language);
@@ -200,32 +196,63 @@ no_results_widget_new (void)
         return widget;
 }
 
+static void
+add_one_language (CcLanguageChooser *chooser,
+                  const char        *locale_id,
+                  gboolean           is_initial)
+{
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
+	GtkWidget *widget;
+	gchar *lang;
+
+	if (!g_str_has_suffix (locale_id, "utf8")) {
+		return;
+	}
+
+	if (!cc_common_language_has_font (locale_id)) {
+		return;
+	}
+
+	if (!gnome_parse_locale (locale_id, &lang, NULL, NULL, NULL)) {
+		return;
+	}
+
+	if (g_hash_table_contains (priv->langs, lang)) {
+		g_free (lang);
+		return;
+	}
+	g_hash_table_add (priv->langs, lang);
+
+	widget = language_widget_new (locale_id, lang, !is_initial);
+	gtk_container_add (GTK_CONTAINER (priv->language_list), widget);
+	g_free (lang);
+}
+
 static void
 add_languages (CcLanguageChooser  *chooser,
                char               **locale_ids,
                GHashTable          *initial)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
+	GHashTableIter iter;
+	gchar *key;
+
+	g_hash_table_iter_init (&iter, initial);
+	while (g_hash_table_iter_next (&iter, (gpointer *)&key, NULL)) {
+		add_one_language (chooser, key, TRUE);
+	}
 
         while (*locale_ids) {
                 const gchar *locale_id;
-                gboolean is_initial;
-                GtkWidget *widget;
 
                 locale_id = *locale_ids;
                 locale_ids ++;
 
-                if (!cc_common_language_has_font (locale_id))
-                        continue;
-
-                is_initial = (g_hash_table_lookup (initial, locale_id) != NULL);
-                widget = language_widget_new (locale_id, !is_initial);
-
-                gtk_container_add (GTK_CONTAINER (priv->language_list), widget);
+		add_one_language (chooser, locale_id, FALSE);
         }
 
         gtk_container_add (GTK_CONTAINER (priv->language_list), priv->more_item);
-        gtk_container_add (GTK_CONTAINER (priv->language_list), priv->no_results);
+        gtk_list_box_set_placeholder (GTK_LIST_BOX (priv->language_list), priv->no_results);
 
         gtk_widget_show_all (priv->language_list);
 }
@@ -244,78 +271,56 @@ add_all_languages (CcLanguageChooser *chooser)
 }
 
 static gboolean
-match_all (gchar       **words,
-           const gchar  *str)
-{
-        gchar **w;
-
-        for (w = words; *w; ++w)
-                if (!strstr (str, *w))
-                        return FALSE;
-
-        return TRUE;
-}
-
-static gboolean
-language_visible (GtkWidget *child,
-                  gpointer   user_data)
+language_visible (GtkListBoxRow *row,
+                  gpointer       user_data)
 {
         CcLanguageChooser *chooser = user_data;
-        CcLanguageChooserPrivate *priv = chooser->priv;
-        gchar *locale_name = NULL;
-        gchar *locale_current_name = NULL;
-        gchar *locale_untranslated_name = NULL;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
         LanguageWidget *widget;
         gboolean visible;
+        GtkWidget *child;
+        const char *search_term;
 
+        child = gtk_bin_get_child (GTK_BIN (row));
         if (child == priv->more_item)
                 return !priv->showing_extra;
 
-        /* We hide this in the after-refilter handler below. */
-        if (child == priv->no_results)
-                return TRUE;
-
         widget = get_language_widget (child);
 
         if (!priv->showing_extra && widget->is_extra)
                 return FALSE;
 
-        if (!priv->filter_words)
+        search_term = gtk_entry_get_text (GTK_ENTRY (priv->filter_entry));
+        if (!search_term || !*search_term)
                 return TRUE;
 
         visible = FALSE;
 
-        locale_name = cc_util_normalize_casefold_and_unaccent (widget->locale_name);
-        visible = match_all (priv->filter_words, locale_name);
+        visible = g_str_match_string (search_term, widget->locale_name, TRUE);
         if (visible)
                 goto out;
 
-        locale_current_name = cc_util_normalize_casefold_and_unaccent (widget->locale_current_name);
-        visible = match_all (priv->filter_words, locale_current_name);
+        visible = g_str_match_string (search_term, widget->locale_current_name, TRUE);
         if (visible)
                 goto out;
 
-        locale_untranslated_name = cc_util_normalize_casefold_and_unaccent (widget->locale_untranslated_name);
-        visible = match_all (priv->filter_words, locale_untranslated_name);
+        visible = g_str_match_string (search_term, widget->locale_untranslated_name, TRUE);
         if (visible)
                 goto out;
 
  out:
-        g_free (locale_untranslated_name);
-        g_free (locale_current_name);
-        g_free (locale_name);
         return visible;
 }
 
 static gint
-sort_languages (GtkWidget *a,
-                GtkWidget *b,
-                gpointer   data)
+sort_languages (GtkListBoxRow *a,
+                GtkListBoxRow *b,
+                gpointer       data)
 {
         LanguageWidget *la, *lb;
 
-        la = get_language_widget (a);
-        lb = get_language_widget (b);
+        la = get_language_widget (gtk_bin_get_child (GTK_BIN (a)));
+        lb = get_language_widget (gtk_bin_get_child (GTK_BIN (b)));
 
         if (la == NULL)
                 return 1;
@@ -323,6 +328,12 @@ sort_languages (GtkWidget *a,
         if (lb == NULL)
                 return -1;
 
+        if (la->is_extra && !lb->is_extra)
+                return 1;
+
+        if (!la->is_extra && lb->is_extra)
+                return -1;
+
         return strcmp (la->locale_name, lb->locale_name);
 }
 
@@ -330,32 +341,25 @@ static void
 filter_changed (GtkEntry        *entry,
                 CcLanguageChooser *chooser)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
-        gchar *filter_contents = NULL;
-
-        g_clear_pointer (&priv->filter_words, g_strfreev);
-
-        filter_contents =
-                cc_util_normalize_casefold_and_unaccent (gtk_entry_get_text (GTK_ENTRY (priv->filter_entry)));
-        if (!filter_contents) {
-                egg_list_box_refilter (EGG_LIST_BOX (priv->language_list));
-                return;
-        }
-        priv->filter_words = g_strsplit_set (g_strstrip (filter_contents), " ", 0);
-        g_free (filter_contents);
-        egg_list_box_refilter (EGG_LIST_BOX (priv->language_list));
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
+        gtk_list_box_invalidate_filter (GTK_LIST_BOX (priv->language_list));
 }
 
 static void
 show_more (CcLanguageChooser *chooser)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
 
         gtk_widget_show (priv->filter_entry);
         gtk_widget_grab_focus (priv->filter_entry);
 
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->scrolled_window),
+					GTK_POLICY_NEVER,
+					GTK_POLICY_AUTOMATIC);
+	gtk_widget_set_valign (GTK_WIDGET (chooser), GTK_ALIGN_FILL);
+
         priv->showing_extra = TRUE;
-        egg_list_box_refilter (EGG_LIST_BOX (priv->language_list));
+        gtk_list_box_invalidate_filter (GTK_LIST_BOX (priv->language_list));
         g_object_notify_by_pspec (G_OBJECT (chooser), obj_props[PROP_SHOWING_EXTRA]);
 }
 
@@ -363,7 +367,7 @@ static void
 set_locale_id (CcLanguageChooser *chooser,
                const gchar       *new_locale_id)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
 
         if (g_strcmp0 (priv->language, new_locale_id) == 0)
                 return;
@@ -377,114 +381,62 @@ set_locale_id (CcLanguageChooser *chooser,
 }
 
 static void
-child_activated (EggListBox        *box,
-                 GtkWidget         *child,
-                 CcLanguageChooser *chooser)
+row_activated (GtkListBox        *box,
+               GtkListBoxRow     *row,
+               CcLanguageChooser *chooser)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
+        GtkWidget *child;
         LanguageWidget *widget;
 
-        if (child == NULL)
-                return;
-
-        if (child == priv->no_results)
+        if (row == NULL)
                 return;
 
+        child = gtk_bin_get_child (GTK_BIN (row));
         if (child == priv->more_item) {
                 show_more (chooser);
-                return;
+        } else {
+                widget = get_language_widget (child);
+                if (widget == NULL)
+                        return;
+                set_locale_id (chooser, widget->locale_id);
         }
-
-        widget = get_language_widget (child);
-        set_locale_id (chooser, widget->locale_id);
-}
-
-typedef struct {
-        gint count;
-        GtkWidget *ignore;
-} CountChildrenData;
-
-static void
-count_visible_children (GtkWidget *widget,
-                        gpointer   user_data)
-{
-        CountChildrenData *data = user_data;
-        if (widget != data->ignore &&
-            gtk_widget_get_child_visible (widget) &&
-            gtk_widget_get_visible (widget))
-                data->count++;
 }
 
 static void
-end_refilter (EggListBox *list_box,
-              gpointer    user_data)
+update_header_func (GtkListBoxRow *child,
+                    GtkListBoxRow *before,
+                    gpointer       user_data)
 {
-        CcLanguageChooser *chooser = CC_LANGUAGE_CHOOSER (user_data);
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        GtkWidget *header;
 
-        CountChildrenData data = { 0 };
-
-        data.ignore = priv->no_results;
-
-        gtk_container_foreach (GTK_CONTAINER (list_box),
-                               count_visible_children, &data);
-
-        gtk_widget_set_visible (priv->no_results, (data.count == 0));
-}
-
-static void
-update_separator_func (GtkWidget **separator,
-                       GtkWidget  *child,
-                       GtkWidget  *before,
-                       gpointer    user_data)
-{
         if (before == NULL)
                 return;
 
-        if (*separator == NULL) {
-                *separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-                g_object_ref_sink (*separator);
-                gtk_widget_show (*separator);
-        }
+        header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+        gtk_list_box_row_set_header (child, header);
+        gtk_widget_show (header);
 }
 
-#define WID(name) ((GtkWidget *) gtk_builder_get_object (builder, name))
-
 static void
 cc_language_chooser_constructed (GObject *object)
 {
-        GtkBuilder *builder;
         CcLanguageChooser *chooser = CC_LANGUAGE_CHOOSER (object);
-        CcLanguageChooserPrivate *priv = chooser->priv;
-        GError *error = NULL;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
 
         G_OBJECT_CLASS (cc_language_chooser_parent_class)->constructed (object);
 
-        builder = gtk_builder_new ();
-        if (gtk_builder_add_from_resource (builder, "/org/gnome/control-center/language-chooser.ui", &error) == 0) {
-                g_object_unref (builder);
-                g_warning ("failed to load language chooser: %s", error->message);
-                g_error_free (error);
-                return;
-        }
-
-        gtk_container_add (GTK_CONTAINER (chooser), WID ("language-chooser"));
-
-        priv->filter_entry = WID ("language-filter-entry");
-        priv->language_list = WID ("language-list");
+	priv->langs = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
         priv->more_item = more_widget_new ();
         priv->no_results = no_results_widget_new ();
 
-        egg_list_box_set_adjustment (EGG_LIST_BOX (priv->language_list),
-                                     gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (WID ("language-scrolledwindow"))));
-
-        egg_list_box_set_sort_func (EGG_LIST_BOX (priv->language_list),
+        gtk_list_box_set_sort_func (GTK_LIST_BOX (priv->language_list),
                                     sort_languages, chooser, NULL);
-        egg_list_box_set_filter_func (EGG_LIST_BOX (priv->language_list),
+        gtk_list_box_set_filter_func (GTK_LIST_BOX (priv->language_list),
                                       language_visible, chooser, NULL);
-        egg_list_box_set_separator_funcs (EGG_LIST_BOX (priv->language_list),
-                                          update_separator_func, chooser, NULL);
-        egg_list_box_set_selection_mode (EGG_LIST_BOX (priv->language_list),
+        gtk_list_box_set_header_func (GTK_LIST_BOX (priv->language_list),
+                                      update_header_func, chooser, NULL);
+        gtk_list_box_set_selection_mode (GTK_LIST_BOX (priv->language_list),
                                          GTK_SELECTION_NONE);
         add_all_languages (chooser);
 
@@ -492,13 +444,8 @@ cc_language_chooser_constructed (GObject *object)
                           G_CALLBACK (filter_changed),
                           chooser);
 
-        g_signal_connect (priv->language_list, "child-activated",
-                          G_CALLBACK (child_activated), chooser);
-
-        g_signal_connect_after (priv->language_list, "refilter",
-                                G_CALLBACK (end_refilter), chooser);
-
-        egg_list_box_refilter (EGG_LIST_BOX (priv->language_list));
+        g_signal_connect (priv->language_list, "row-activated",
+                          G_CALLBACK (row_activated), chooser);
 
         if (priv->language == NULL)
                 priv->language = cc_common_language_get_current_language ();
@@ -506,6 +453,17 @@ cc_language_chooser_constructed (GObject *object)
         sync_all_checkmarks (chooser);
 }
 
+static void
+cc_language_chooser_finalize (GObject *object)
+{
+	CcLanguageChooser *chooser = CC_LANGUAGE_CHOOSER (object);
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
+
+	g_hash_table_unref (priv->langs);
+
+	G_OBJECT_CLASS (cc_language_chooser_parent_class)->finalize (object);
+}
+
 static void
 cc_language_chooser_get_property (GObject      *object,
                                   guint         prop_id,
@@ -543,25 +501,20 @@ cc_language_chooser_set_property (GObject      *object,
         }
 }
 
-static void
-cc_language_chooser_finalize (GObject *object)
-{
-        CcLanguageChooser *chooser = CC_LANGUAGE_CHOOSER (object);
-        CcLanguageChooserPrivate *priv = chooser->priv;
-
-        g_strfreev (priv->filter_words);
-}
-
 static void
 cc_language_chooser_class_init (CcLanguageChooserClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        g_type_class_add_private (object_class, sizeof(CcLanguageChooserPrivate));
+        gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/org/gnome/control-center/language-chooser.ui");
 
+        gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), CcLanguageChooser, filter_entry);
+        gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), CcLanguageChooser, language_list);
+        gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), CcLanguageChooser, scrolled_window);
+
+	object_class->finalize = cc_language_chooser_finalize;
         object_class->get_property = cc_language_chooser_get_property;
         object_class->set_property = cc_language_chooser_set_property;
-        object_class->finalize = cc_language_chooser_finalize;
         object_class->constructed = cc_language_chooser_constructed;
 
         obj_props[PROP_LANGUAGE] =
@@ -575,23 +528,23 @@ cc_language_chooser_class_init (CcLanguageChooserClass *klass)
         g_object_class_install_properties (object_class, PROP_LAST, obj_props);
 }
 
-void
+static void
 cc_language_chooser_init (CcLanguageChooser *chooser)
 {
-        chooser->priv = GET_PRIVATE (chooser);
+        gtk_widget_init_template (GTK_WIDGET (chooser));
 }
 
 void
 cc_language_chooser_clear_filter (CcLanguageChooser *chooser)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
         gtk_entry_set_text (GTK_ENTRY (priv->filter_entry), "");
 }
 
 const gchar *
 cc_language_chooser_get_language (CcLanguageChooser *chooser)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
         return priv->language;
 }
 
@@ -605,6 +558,6 @@ cc_language_chooser_set_language (CcLanguageChooser *chooser,
 gboolean
 cc_language_chooser_get_showing_extra (CcLanguageChooser *chooser)
 {
-        CcLanguageChooserPrivate *priv = chooser->priv;
+        CcLanguageChooserPrivate *priv = cc_language_chooser_get_instance_private (chooser);
         return priv->showing_extra;
 }
