@@ -13,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
@@ -34,8 +32,6 @@
 
 #include <gtk/gtk.h>
 
-#include <egg-list-box.h>
-
 #include <nm-client.h>
 #include <nm-device-wifi.h>
 #include <nm-access-point.h>
@@ -53,6 +49,11 @@ typedef enum {
 } NMAccessPointSecurity;
 
 struct _GisNetworkPagePrivate {
+  GtkWidget *network_list;
+  GtkWidget *scrolled_window;
+  GtkWidget *no_network_label;
+  GtkWidget *no_network_spinner;
+
   NMClient *nm_client;
   NMRemoteSettings *nm_settings;
   NMDevice *nm_device;
@@ -63,9 +64,6 @@ typedef struct _GisNetworkPagePrivate GisNetworkPagePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GisNetworkPage, gis_network_page, GIS_TYPE_PAGE);
 
-#define OBJ(type,name) ((type)gtk_builder_get_object(GIS_PAGE(page)->builder,(name)))
-#define WID(name) OBJ(GtkWidget*,name)
-
 static GPtrArray *
 get_strongest_unique_aps (const GPtrArray *aps)
 {
@@ -148,33 +146,37 @@ get_access_point_security (NMAccessPoint *ap)
 }
 
 static gint
-ap_sort (gconstpointer a, gconstpointer b, gpointer data)
+ap_sort (GtkListBoxRow *a,
+         GtkListBoxRow *b,
+         gpointer data)
 {
-        guint sa, sb;
+  GtkWidget *wa, *wb;
+  guint sa, sb;
 
-        sa = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (a), "strength"));
-        sb = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (b), "strength"));
-        if (sa > sb) return -1;
-        if (sb > sa) return 1;
+  wa = gtk_bin_get_child (GTK_BIN (a));
+  wb = gtk_bin_get_child (GTK_BIN (b));
 
-        return 0;
+  sa = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (wa), "strength"));
+  sb = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (wb), "strength"));
+  if (sa > sb) return -1;
+  if (sb > sa) return 1;
+
+  return 0;
 }
 
 static void
-update_separator (GtkWidget **separator,
-                  GtkWidget  *child,
-                  GtkWidget  *before,
-                  gpointer    user_data)
+update_header_func (GtkListBoxRow *child,
+                    GtkListBoxRow *before,
+                    gpointer       user_data)
 {
+  GtkWidget *header;
+
   if (before == NULL)
     return;
 
-  if (*separator == NULL)
-    {
-      *separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-      gtk_widget_show (*separator);
-      g_object_ref_sink (*separator);
-    }
+  header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+  gtk_list_box_row_set_header (child, header);
+  gtk_widget_show (header);
 }
 
 static void
@@ -191,7 +193,6 @@ add_access_point (GisNetworkPage *page, NMAccessPoint *ap, NMAccessPoint *active
   GtkWidget *row;
   GtkWidget *widget;
   GtkWidget *box;
-  GtkWidget *list;
 
   ssid = nm_access_point_get_ssid (ap);
   object_path = nm_object_get_path (NM_OBJECT (ap));
@@ -230,8 +231,8 @@ add_access_point (GisNetworkPage *page, NMAccessPoint *ap, NMAccessPoint *active
   strength = nm_access_point_get_strength (ap);
 
   row = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
-  gtk_widget_set_margin_left (row, 12);
-  gtk_widget_set_margin_right (row, 12);
+  gtk_widget_set_margin_start (row, 12);
+  gtk_widget_set_margin_end (row, 12);
   widget = gtk_label_new (ssid_text);
   gtk_widget_set_margin_top (widget, 12);
   gtk_widget_set_margin_bottom (widget, 12);
@@ -288,20 +289,19 @@ add_access_point (GisNetworkPage *page, NMAccessPoint *ap, NMAccessPoint *active
   g_object_set_data (G_OBJECT (row), "ssid", (gpointer) ssid);
   g_object_set_data (G_OBJECT (row), "strength", GUINT_TO_POINTER (strength));
 
-  list = WID ("network-list");
-  gtk_container_add (GTK_CONTAINER (list), row);
+  gtk_container_add (GTK_CONTAINER (priv->network_list), row);
 }
 
 static void
 add_access_point_other (GisNetworkPage *page)
 {
+  GisNetworkPagePrivate *priv = gis_network_page_get_instance_private (page);
   GtkWidget *row;
   GtkWidget *widget;
-  GtkWidget *list;
 
   row = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
-  gtk_widget_set_margin_left (row, 12);
-  gtk_widget_set_margin_right (row, 12);
+  gtk_widget_set_margin_start (row, 12);
+  gtk_widget_set_margin_end (row, 12);
   widget = gtk_label_new (C_("Wireless access point", "Other…"));
   gtk_widget_set_margin_top (widget, 12);
   gtk_widget_set_margin_bottom (widget, 12);
@@ -311,8 +311,7 @@ add_access_point_other (GisNetworkPage *page)
   g_object_set_data (G_OBJECT (row), "object-path", "ap-other...");
   g_object_set_data (G_OBJECT (row), "strength", GUINT_TO_POINTER (0));
 
-  list = WID("network-list");
-  gtk_container_add (GTK_CONTAINER (list), row);
+  gtk_container_add (GTK_CONTAINER (priv->network_list), row);
 }
 
 static void refresh_wireless_list (GisNetworkPage *page);
@@ -325,85 +324,42 @@ refresh_again (gpointer user_data)
   return FALSE;
 }
 
-static void
-refresh_without_device (GisNetworkPage *page)
-{
-  GisNetworkPagePrivate *priv = gis_network_page_get_instance_private (page);
-  GtkWidget *label;
-  GtkWidget *spinner;
-  GtkWidget *swin;
-
-  swin = WID("network-scrolledwindow");
-  label = WID("no-network-label");
-  spinner = WID("no-network-spinner");
-
-  if (nm_client_get_state (priv->nm_client) == NM_STATE_CONNECTED_GLOBAL)
-    ;
-  else if (priv->nm_device != NULL)
-    gtk_label_set_text (GTK_LABEL (label), _("Network is not available."));
-  else
-    gtk_label_set_text (GTK_LABEL (label), _("No network devices found."));
-
-  gtk_widget_hide (swin);
-  gtk_widget_hide (spinner);
-  gtk_widget_show (label);
-}
-
 static void
 refresh_wireless_list (GisNetworkPage *page)
 {
   GisNetworkPagePrivate *priv = gis_network_page_get_instance_private (page);
-  NMDeviceState state = NM_DEVICE_STATE_UNAVAILABLE;
   NMAccessPoint *active_ap = NULL;
   NMAccessPoint *ap;
   const GPtrArray *aps;
   GPtrArray *unique_aps;
   guint i;
-  GtkWidget *label;
-  GtkWidget *spinner;
-  GtkWidget *swin;
   GList *children, *l;
-  GtkWidget *list;
 
   priv->refreshing = TRUE;
 
-  if (NM_IS_DEVICE_WIFI (priv->nm_device)) {
-    state = nm_device_get_state (priv->nm_device);
+  g_assert (NM_IS_DEVICE_WIFI (priv->nm_device));
 
-    active_ap = nm_device_wifi_get_active_access_point (NM_DEVICE_WIFI (priv->nm_device));
+  active_ap = nm_device_wifi_get_active_access_point (NM_DEVICE_WIFI (priv->nm_device));
 
-    list = WID ("network-list");
-    children = gtk_container_get_children (GTK_CONTAINER (list));
-    for (l = children; l; l = l->next) {
-      gtk_container_remove (GTK_CONTAINER (list), l->data);
-    }
-    g_list_free (children);
+  children = gtk_container_get_children (GTK_CONTAINER (priv->network_list));
+  for (l = children; l; l = l->next)
+    gtk_container_remove (GTK_CONTAINER (priv->network_list), l->data);
+  g_list_free (children);
 
-    aps = nm_device_wifi_get_access_points (NM_DEVICE_WIFI (priv->nm_device));
-  }
+  aps = nm_device_wifi_get_access_points (NM_DEVICE_WIFI (priv->nm_device));
 
-  swin = WID("network-scrolledwindow");
-  label = WID("no-network-label");
-  spinner = WID("no-network-spinner");
-
-  if (state == NM_DEVICE_STATE_UNMANAGED ||
-      state == NM_DEVICE_STATE_UNAVAILABLE) {
-    refresh_without_device (page);
-    goto out;
-  }
-  else if (aps == NULL || aps->len == 0) {
-    gtk_label_set_text (GTK_LABEL (label), _("Checking for available wireless networks"));
-    gtk_widget_hide (swin);
-    gtk_widget_show (spinner);
-    gtk_widget_show (label);
+  if (aps == NULL || aps->len == 0) {
+    gtk_label_set_text (GTK_LABEL (priv->no_network_label), _("Checking for available wireless networks"));
+    gtk_widget_show (priv->no_network_spinner);
+    gtk_widget_show (priv->no_network_label);
+    gtk_widget_hide (priv->scrolled_window);
     g_timeout_add_seconds (1, refresh_again, page);
 
     goto out;
-  }
-  else {
-    gtk_widget_show (swin);
-    gtk_widget_hide (spinner);
-    gtk_widget_hide (label);
+  } else {
+    gtk_widget_hide (priv->no_network_spinner);
+    gtk_widget_hide (priv->no_network_label);
+    gtk_widget_show (priv->scrolled_window);
   }
 
   unique_aps = get_strongest_unique_aps (aps);
@@ -418,15 +374,6 @@ refresh_wireless_list (GisNetworkPage *page)
   priv->refreshing = FALSE;
 }
 
-static void
-device_state_changed (NMDevice   *device,
-                      GParamSpec *pspec,
-                      gpointer    user_data)
-{
-  GisNetworkPage *page = GIS_NETWORK_PAGE (user_data);
-  refresh_wireless_list (page);
-}
-
 static void
 connection_activate_cb (NMClient *client,
                         NMActiveConnection *connection,
@@ -461,7 +408,9 @@ connect_to_hidden_network (GisNetworkPage *page)
 }
 
 static void
-child_activated (EggListBox *box, GtkWidget *child, GisNetworkPage *page)
+row_activated (GtkListBox *box,
+               GtkListBoxRow *row,
+               GisNetworkPage *page)
 {
   GisNetworkPagePrivate *priv = gis_network_page_get_instance_private (page);
   gchar *object_path;
@@ -471,10 +420,12 @@ child_activated (EggListBox *box, GtkWidget *child, GisNetworkPage *page)
   NMSettingWireless *setting;
   const GByteArray *ssid_target;
   const GByteArray *ssid;
+  GtkWidget *child;
 
   if (priv->refreshing)
     return;
 
+  child = gtk_bin_get_child (GTK_BIN (row));
   object_path = g_object_get_data (G_OBJECT (child), "object-path");
   ssid_target = g_object_get_data (G_OBJECT (child), "ssid");
 
@@ -550,6 +501,22 @@ active_connections_changed (NMClient *client, GParamSpec *pspec, GisNetworkPage
   refresh_wireless_list (page);
 }
 
+static void
+sync_complete (GisNetworkPage *page)
+{
+  GisNetworkPagePrivate *priv = gis_network_page_get_instance_private (page);
+  gboolean activated;
+
+  activated = (nm_device_get_state (priv->nm_device) == NM_DEVICE_STATE_ACTIVATED);
+  gis_page_set_complete (GIS_PAGE (page), activated);
+}
+
+static void
+device_state_changed (GObject *object, GParamSpec *param, GisNetworkPage *page)
+{
+  sync_complete (page);
+}
+
 static void
 gis_network_page_constructed (GObject *object)
 {
@@ -558,22 +525,14 @@ gis_network_page_constructed (GObject *object)
   const GPtrArray *devices;
   NMDevice *device;
   guint i;
-  DBusGConnection *bus;
-  GError *error;
-  gboolean visible = TRUE;
-  GtkWidget *box;
+  gboolean visible = FALSE;
 
   G_OBJECT_CLASS (gis_network_page_parent_class)->constructed (object);
 
-  gtk_container_add (GTK_CONTAINER (page), WID ("network-page"));
-
   priv->icons = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
   priv->nm_client = nm_client_new ();
 
-  g_signal_connect (priv->nm_client, "notify::active-connections",
-                    G_CALLBACK (active_connections_changed), page);
-
   devices = nm_client_get_devices (priv->nm_client);
   if (devices) {
     for (i = 0; i < devices->len; i++) {
@@ -585,43 +544,35 @@ gis_network_page_constructed (GObject *object)
       if (nm_device_get_device_type (device) == NM_DEVICE_TYPE_WIFI) {
         /* FIXME deal with multiple, dynamic devices */
         priv->nm_device = g_object_ref (device);
-        g_signal_connect (G_OBJECT (device), "notify::state",
-                          G_CALLBACK (device_state_changed), page);
         break;
       }
     }
   }
 
-  if (priv->nm_device == NULL) {
-    visible = FALSE;
-    refresh_without_device (page);
+  if (priv->nm_device == NULL)
     goto out;
-  }
 
-  error = NULL;
-  bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
-  if (!bus) {
-    g_warning ("Error connecting to system D-Bus: %s",
-               error->message);
-    g_error_free (error);
-    visible = FALSE;
+  if (nm_device_get_state (priv->nm_device) == NM_DEVICE_STATE_ACTIVATED)
     goto out;
-  }
-  priv->nm_settings = nm_remote_settings_new (bus);
 
-  box = WID ("network-list");
+  visible = TRUE;
+  priv->nm_settings = nm_remote_settings_new (NULL);
 
-  egg_list_box_set_selection_mode (EGG_LIST_BOX (box), GTK_SELECTION_NONE);
-  egg_list_box_set_separator_funcs (EGG_LIST_BOX (box), update_separator, NULL, NULL);
-  egg_list_box_set_sort_func (EGG_LIST_BOX (box), ap_sort, NULL, NULL);
-  egg_list_box_set_adjustment (EGG_LIST_BOX (box), gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (WID ("network-scrolledwindow"))));
+  g_signal_connect (priv->nm_device, "notify::state",
+                    G_CALLBACK (device_state_changed), page);
+  g_signal_connect (priv->nm_client, "notify::active-connections",
+                    G_CALLBACK (active_connections_changed), page);
 
-  g_signal_connect (box, "child-activated",
-                    G_CALLBACK (child_activated), page);
+  gtk_list_box_set_selection_mode (GTK_LIST_BOX (priv->network_list), GTK_SELECTION_NONE);
+  gtk_list_box_set_header_func (GTK_LIST_BOX (priv->network_list), update_header_func, NULL, NULL);
+  gtk_list_box_set_sort_func (GTK_LIST_BOX (priv->network_list), ap_sort, NULL, NULL);
+  g_signal_connect (priv->network_list, "row-activated",
+                    G_CALLBACK (row_activated), page);
 
   refresh_wireless_list (page);
+  sync_complete (page);
 
-  gis_page_set_complete (GIS_PAGE (page), TRUE);
+  gis_page_set_skippable (GIS_PAGE (page), TRUE);
 
  out:
   gtk_widget_set_visible (GTK_WIDGET (page), visible);
@@ -653,6 +604,13 @@ gis_network_page_class_init (GisNetworkPageClass *klass)
   GisPageClass *page_class = GIS_PAGE_CLASS (klass);
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/org/gnome/initial-setup/gis-network-page.ui");
+
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisNetworkPage, network_list);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisNetworkPage, scrolled_window);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisNetworkPage, no_network_label);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisNetworkPage, no_network_spinner);
+
   page_class->page_id = PAGE_ID;
   page_class->locale_changed = gis_network_page_locale_changed;
   object_class->constructed = gis_network_page_constructed;
@@ -663,6 +621,8 @@ static void
 gis_network_page_init (GisNetworkPage *page)
 {
   g_resources_register (network_get_resource ());
+
+  gtk_widget_init_template (GTK_WIDGET (page));
 }
 
 void
